/*
 * Copyright 2011, Maarten Baert <maarten-baert@hotmail.com>
 * 
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#pragma once

#include <limits>

// function exported to Game Maker
#define gmexport extern "C" __declspec (dllexport)

// gm_cast - cast without throwing exceptions
template<typename A>
inline A gm_cast(double d) {
	A minval = std::numeric_limits<A>::min();
	A maxval = std::numeric_limits<A>::max();
	if(d<(double)(minval)) {
		return minval;
	}
	if(d>(double)(maxval)) {
		return maxval;
	}
	return (A)(d);
}
template<>
inline float gm_cast<float>(double d) {
	float minval = -std::numeric_limits<float>::max();
	float maxval = std::numeric_limits<float>::max();
	if(d<(double)(minval)) {
		return minval;
	}
	if(d>(double)(maxval)) {
		return maxval;
	}
	return (float)(d);
}
template<>
inline double gm_cast<double>(double d) {
	return d;
}
template<>
inline bool gm_cast<bool>(double d) {
	return (d>=0.5);
}

